/*
 *  Vazaar
 *  file: vazaar.c
 *
 *
 *  Copyright (C) 2005 Tomás Vírseda <kaskaras@yahoo.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gnome.h>
#include <gtk/gtk.h>
#include <libxml/xmlreader.h>
#include <gnet.h>
#include <glade/glade.h>
#include <gtkmozembed.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gconf/gconf-client.h>

#include "vazaar.h"
#include "callbacks.h"
#include "debug.h"
#include "items.h"

extern xmlDocPtr vazaar_doc;
extern xmlDocPtr cats_doc;
extern xmlDocPtr items_doc;

/* Drop targets. */
enum {
	TARGET_STRING,
	TARGET_URI_LIST
};

static const GtkTargetEntry drop_target_types[] = {
	{ "STRING",     0, TARGET_STRING },
	{ "text/plain", 0, TARGET_STRING },
	{ "text/uri-list", 0, TARGET_URI_LIST },
};

/* Stolen from GLib 2.6: g_uri_list_extract_uris (const gchar *uri_list) */
/* Stolen from planner */
static gchar **
uri_list_extract_uris (const gchar *uri_list)
{
  GSList *uris, *u;
  const gchar *p, *q;
  gchar **result;
  gint n_uris = 0;

  uris = NULL;

  p = uri_list;

  /* We don't actually try to validate the URI according to RFC
   * 2396, or even check for allowed characters - we just ignore
   * comments and trim whitespace off the ends.  We also
   * allow LF delimination as well as the specified CRLF.
   *
   * We do allow comments like specified in RFC 2483.
   */
  while (p)
    {
      if (*p != '#')
	{
	  while (g_ascii_isspace (*p))
	    p++;

	  q = p;
	  while (*q && (*q != '\n') && (*q != '\r'))
	    q++;

	  if (q > p)
	    {
	      q--;
	      while (q > p && g_ascii_isspace (*q))
		q--;

	      if (q > p)
		{
		  uris = g_slist_prepend (uris, g_strndup (p, q - p + 1));
		  n_uris++;
		}
	    }
	}
      p = strchr (p, '\n');
      if (p)
	p++;
    }

  result = g_new (gchar *, n_uris + 1);

  result[n_uris--] = NULL;
  for (u = uris; u; u = u->next)
    result[n_uris--] = u->data;

  g_slist_free (uris);

  return result;
}

void
vazaar_gui_vazaar_init_drag_and_drop (Vazaar *this) {
	gtk_drag_dest_set (GTK_WIDGET (this->frmitems),
			   GTK_DEST_DEFAULT_ALL,
			   drop_target_types,
			   G_N_ELEMENTS (drop_target_types),
			   GDK_ACTION_COPY);
	
	g_signal_connect	(	this->frmitems, 
							"drag_data_received",
							G_CALLBACK (treeitem_drag_data_received_cb), 
							this);
}

static void
treeitem_drag_data_received_cb	(	GtkWidget			*widget,
									GdkDragContext		*context,
									int					x,
							  		int					y,
									GtkSelectionData	*data,
									guint				info,
									guint				time,
									Vazaar				*app) {
	gchar	**uris;
	gint	i;
										
	if (data->length < 0 || data->format != 8) {
		g_message ("Don't know how to handle format %d", data->format);
		gtk_drag_finish (context, FALSE, FALSE, time);
		return;
	}
	
	uris = uri_list_extract_uris (data->data);

	i = 0;
	/* At the moment, only one uri is added as item */
	/*
	while (uris[i]) {
		g_message ("%s", uris[i]);
		i++;
	}
	*/
	
	vazaar_gui_druiditem_new (app, NULL, uris[0]);
	
	g_strfreev (uris);

	gtk_drag_finish (context, TRUE, FALSE, time);
}

Vazaar         *
vazaar_gui_vazaar_new()
{
	GError			*error = NULL;
    Vazaar         *this;
	gchar			*path;
	const gchar		*imgc, *imge;
	GtkWidget		*imgexpandcat,
					*imgcollapsecat;
	
    this = g_new0(Vazaar, 1);
	//this->client	= gconf_client_get_default ();
    this->xml 		= glade_xml_new(GLADE_VAZAAR_PROJECT_FILE, GLADE_VAZAAR_MAIN_WIN, NULL);
    this->window 	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_MAIN_WIN);
	this->frmwastebasket = glade_xml_get_widget(this->xml, GLADE_VAZAAR_WASTEBASKET_CONTAINER);
    this->treecat 	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_TREEVIEW_CATEGORIES);
    this->treeitem	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_TREEVIEW_ITEMS);
    this->hpcat 	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_HPANED);
    this->vpitem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_VPANED);
    this->statusbar = glade_xml_get_widget (this->xml, GLADE_VAZAAR_STATUSBAR);
	this->frmitems	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_FRMITEMS);
	this->frmcats	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_FRMCATS);
	
    this->addcat	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_ADDCAT);
    this->editcat	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_EDITCAT);
    this->delcat	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_DELCAT);
	this->expcat	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_EXPCAT);
	this->collcat	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_COLLCAT);
	this->wastebasket	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_WASTEBASKET);
	this->emptybasket	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_EMPTY_WASTEBASKET);
	
    this->additem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_ADDITEM);
    this->edititem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_EDITITEM);
    this->delitem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_DELITEM);
	this->infoitem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_INFOITEM);
	this->firstitem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_FIRSTITEM);
	this->previtem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_PREVITEM);
	this->refreshitem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_REFRESHITEM);
	this->nextitem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_NEXTITEM);
	this->lastitem	= glade_xml_get_widget(this->xml, GLADE_VAZAAR_TLB_LASTITEM);
	this->viewcat	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_TLB_VIEWCAT);
	this->fullscreen = glade_xml_get_widget (this->xml, GLADE_VAZAAR_TLB_FULLSCREEN);
	g_assert (this->fullscreen);
	/* Panel Item Info */
	this->frameInfo = glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_FRAME);
	this->itemInfoIcon	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_ICON);
	this->itemInfoId	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_ID);
	this->itemInfoName	= glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_NAME);
	this->itemInfoDesc = glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_DESC);
	this->itemInfoMIME = glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_MIME);
	this->itemInfoApp = glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_DEFAPP);
	this->itemInfoLaunch = glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_LAUNCH);
	this->itemInfoSDate = glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_SDATE);
	this->itemInfoMDate = glade_xml_get_widget (this->xml, GLADE_VAZAAR_INFOITEM_MDATE);
	this->mime_icon_cache = NULL;
    this->vboxitems = glade_xml_get_widget(this->xml, GLADE_VAZAAR_VBOX_ITEMS);
	
	this->force_reload = FALSE;
	this->citem = g_strdup_printf ("-");
	this->current_items = NULL;
	/* Initializing Drag & Drop */
	vazaar_gui_vazaar_init_drag_and_drop (this);
	 
	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (this->treecat), TRUE);
	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (this->treeitem), TRUE);
	
	imgc = g_strdup_printf ("%s/%s", PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/system/", "fold_open.png");
	imge = g_strdup_printf ("%s/%s", PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/system/", "fold_close.png");
	imgcollapsecat = gtk_image_new_from_file (imgc);
	imgexpandcat = gtk_image_new_from_file (imge);
	gtk_button_set_image (GTK_BUTTON(this->expcat), imgexpandcat);
	gtk_button_set_image (GTK_BUTTON(this->collcat), imgcollapsecat);
	path = g_strdup_printf("%s/%s", PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/system/", "gnome-vazaar.svg");
	gtk_window_set_icon_from_file (GTK_WINDOW(this->window), path, &error);
	if (error) {
		g_message ("%s", error->message);
	}
	g_free (path);
    /*
     * Connect all signals 
     */
    vazaar_gui_vazaar_connect_all_signals(this);
	
	/* Select category: important ??? */ 
    return this;
}

void
vazaar_gui_vazaar_destroy(Vazaar * this)
{
	g_return_if_fail (this);
	
	gnome_vfs_shutdown();
	gtk_main_quit();
	if (DEBUG) {
	    g_message(_("Program terminated succesfully."));
		g_mem_profile();
	}
}

void
vazaar_gui_vazaar_show(Vazaar * this)
{
    g_return_if_fail(this);

    gtk_widget_show(this->window);
}

void
vazaar_gui_vazaar_connect_all_signals(Vazaar * this)
{
    g_return_if_fail(this != NULL);
	
    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_QUIT_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_mainWin_delete_event),
				  						this);
	
    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_ADDCAT_SIGNAL, 
				  						GTK_SIGNAL_FUNC(on_add_category_activate),
				  						this);	

    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_EDITCAT_SIGNAL, 
				  						GTK_SIGNAL_FUNC(on_edit_category_activate),
				  						this);	
	
	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_DELCAT_SIGNAL, 
										GTK_SIGNAL_FUNC(on_delete_category_activate),
										this);	
						
	
    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_ADDITEM_SIGNAL, 
				  						GTK_SIGNAL_FUNC(on_add_item_activate),
				  						this);	

    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_EDITITEM_SIGNAL, 
				  						GTK_SIGNAL_FUNC(on_edit_item_activate),
				  						this);	
	
	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_DELITEM_SIGNAL, 
										GTK_SIGNAL_FUNC(on_delete_item_activate),
										this);	
										
    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_INFOITEM_SIGNAL, 
				  						GTK_SIGNAL_FUNC(on_show_item_info_activate),
				  						this);	
	
	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_SHOWCATS_SIGNAL, 
										GTK_SIGNAL_FUNC(on_show_categories_panel_activate),
										this);														

    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_ABOUT_SIGNAL, 
				  						GTK_SIGNAL_FUNC(on_about_activate),
				  						this);	
	
    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_MENU_QUIT_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_mainWin_delete_event),
				  						this);
	
    glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_BTNVIEWCAT_TOGGLED_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_tgbViewCat_toggled),
				  						this);
	
    glade_xml_signal_connect_data	(	this->xml, VAZAAR_CONFIGURE_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_mainWin_configure_event), 
										this);
	
    glade_xml_signal_connect_data	(	this->xml,
				  						VAZAAR_TREECAT_ROW_CHANGED_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_trvVazaarCat_cursor_changed), 
										this);
	/*									
    glade_xml_signal_connect_data	(	this->xml,
				  						VAZAAR_TREECAT_ROW_CHANGED_SIGNAL,
				  						GTK_SIGNAL_FUNC (on_trvVazaarCat_row_activated), 
										this);
	*/		  
	/* Item Toolbar Signals */
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_BTNADDITEM_CLICKED_SIGNAL,
										GTK_SIGNAL_FUNC(on_tlbItemAdd_clicked),
										this);
										
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_BTNEDITITEM_CLICKED_SIGNAL,
										GTK_SIGNAL_FUNC(on_tlbItemEdit_clicked),
										this);
										
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_BTNDELITEM_CLICKED_SIGNAL,
										GTK_SIGNAL_FUNC(on_tlbItemDel_clicked),
										this);
										
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_BTNINFOITEM_CLICKED_SIGNAL,
										GTK_SIGNAL_FUNC(on_tgbItemInfo_toggled),
										this);

    glade_xml_signal_connect_data	(	this->xml,
				  						VAZAAR_BTNADDCAT_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_tlbAddCat_clicked),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml,
				  						VAZAAR_BTNEDITCAT_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_tlbEditCat_clicked),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml,
				  						VAZAAR_BTNDELCAT_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_tlbDelCat_clicked),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml,
				  						VAZAAR_BTNEXPCAT_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_tlbExpCat_clicked),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml,
				  						VAZAAR_BTNCOLCAT_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC(on_tlbColCat_clicked),
				  						this);
										
	glade_xml_signal_connect_data	(	this->xml, 
										TRVITEM_CURSOR_CHANGED_SIGNAL,
				  						GTK_SIGNAL_FUNC(vazaar_gui_treeitem_row_changed), 
										this);

	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_BTNFIRSTITEM_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC (on_tlbItemFirst_clicked), 
										this);
										
	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_BTNPREVIOUSITEM_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC (on_tlbItemPrevious_clicked), 
										this);
										
	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_BTNREFRESITEM_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC (on_tlbItemRefresh_clicked), 
										this);
										
	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_BTNNEXTITEM_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC (on_tlbItemNext_clicked), 
										this);
										
	glade_xml_signal_connect_data	(	this->xml, 
										VAZAAR_BTNLASTITEM_CLICKED_SIGNAL,
				  						GTK_SIGNAL_FUNC (on_tlbItemLast_clicked), 
										this);
																				
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_ITEM_LAUNCH_DEFAPP_SIGNAL,
										GTK_SIGNAL_FUNC(vazaar_cmd_item_launch_defapp),
										this);
										
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_INFOPANEL_EXPANDER_ACTIVATED,
										GTK_SIGNAL_FUNC(vazaar_gui_infopanel_activated),
										this);

    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_WASTEBASKET_TOGGLED_SIGNAL,
										GTK_SIGNAL_FUNC(vazaar_gui_wastebasket_toggled),
										this);
										
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_EMPTYBASKET_CLICKED_SIGNAL,
										GTK_SIGNAL_FUNC(vazaar_gui_emptybasket_clicked),
										this);
		
    glade_xml_signal_connect_data	(	this->xml,
										VAZAAR_BTNFULLSCREEN_TOGGLE_SIGNAL,
										GTK_SIGNAL_FUNC(vazaar_gui_fullscreen_mode),
										this);		
	
}

gchar *
vazaar_xml_vazaardoc_get () {
	// $HOME/.vazaar/vazaar.xml
	return g_strdup_printf("%s/vazaar.xml", vazaar_cmd_appdir_get());	
}

xmlNodePtr
vazaar_xml_vazaar_get_root()
{
    gchar		*doc;
	xmlDocPtr	vazaar_doc;
	xmlNodePtr	vazaar_root;
	
    doc = g_strdup_printf("%s/vazaar.xml", vazaar_cmd_appdir_get());	// $HOME/.vazaar/vazaar.xml
	vazaar_doc = xmlNewDoc("1.0");
	vazaar_root = xmlNewDocNode(vazaar_doc, NULL, "vazaar", NULL);
	xmlDocSetRootElement(vazaar_doc, vazaar_root);
	
    return vazaar_root;
}

gchar *
vazaar_xml_catdoc_get()
{
	// $HOME/.vazaar/categories.xml
    return  g_strdup_printf("%s/categories.xml", vazaar_cmd_appdir_get());	
}

gchar *
vazaar_xml_itemdoc_get()
{
	// $HOME/.vazaar/items.xml
	return g_strdup_printf("%s/items.xml", vazaar_cmd_appdir_get());	
}


int
vazaar_xml_vazaar_savefile()
{
    gchar          *doc_path;
    int             res;

    doc_path = vazaar_xml_vazaardoc_get();
    res = xmlSaveFile(doc_path, vazaar_doc);
    g_free(doc_path);
    return res;
}

gchar          *
vazaar_cmd_item_md5sum_get(GString * item)
{
    GMD5           *md5;
    md5 = gnet_md5_new(item->str, item->len);
    return gnet_md5_get_string(md5);
}

/*
 * Devuelve un pixbuf a escala que encaje en el contenedor cuyas medidas
 * especificamos... en teoria.
 */
GdkPixbuf      *
vazaar_cmd_pixbuf_scale_get(GdkPixbuf * pixbuf, gint wdg_width,
			    gint wdg_height)
{
    GdkPixbuf      *pixbuf_scaled = NULL;
    gint            pxb_width,	// pixbuf width
                    pxb_height;	// pixbuf height
    gfloat          scale = 0.0;

    pxb_width = gdk_pixbuf_get_width(pixbuf);
    pxb_height = gdk_pixbuf_get_height(pixbuf);

    if (DEBUG)
	g_message("%s =>\npixbuf(%d,%d)\nwidget(%d,%d)", __FUNCTION__,
		  pxb_width, pxb_height, wdg_width, wdg_height);

    if ((pxb_width < wdg_width) && (pxb_height < wdg_height))
		scale = 1.0;

    if ((pxb_width > wdg_width) && (pxb_height > wdg_height)) {
		if (pxb_width < pxb_height) {
			scale = (wdg_height * (1.0) / (pxb_height) * (1.0));
		} 
		else {
			scale = (wdg_width * (1.0)) / (pxb_width * (1.0));
		}
	}

    if (pxb_width > wdg_width) {
		scale = (wdg_width * (1.0)) / (pxb_width * (1.0));
    }

    if (pxb_height > wdg_height) {
		scale = (wdg_height * (1.0) / (pxb_height) * (1.0));
    }

    pxb_width = (gint)(pxb_width * scale);
    pxb_height = (gint)(pxb_height * scale);
    pixbuf_scaled =
	gdk_pixbuf_scale_simple(pixbuf, pxb_width, pxb_height,
				GDK_INTERP_BILINEAR);

    if (DEBUG)
		g_message("%s => scale = %f", __FUNCTION__, scale);

    return pixbuf_scaled;
}

gchar			*vazaar_cmd_item_get_mime_supertype (const gchar *mime) {
	gchar	*supermime;
	gint	 i = 0;
	supermime = g_strdup (mime);
	while (supermime[i] != '/') {
		i++;
	}
	supermime[i] = '\0';
	
	return supermime;
}


const gchar    *
vazaar_cmd_appdir_get(void)
{
    return g_strdup_printf("%s/.vazaar", g_get_home_dir());	// $HOME/.vazaar
}

const gchar    *
vazaar_cmd_storedir_get(void)
{
    return g_strdup_printf("%s/store", vazaar_cmd_appdir_get());	// $HOME/.vazaar/store
}

gint
vazaar_gui_show_dialog (GtkWidget * window, gchar *stock_id, gchar *title, gchar *message)
{
	gint			 result;
	GtkWidget		*dlg;
	
	dlg = vazaar_hig_dialog_new (GTK_WINDOW (window), stock_id, title, message, TRUE); 
	
	if (vazaar_cmd_strings_compare (stock_id, "gtk-dialog-error") == 0) {
		gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
	}
	else if (vazaar_cmd_strings_compare (stock_id, "gtk-dialog-info") == 0) {
		gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
	}
	else {		
		gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
		gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
		gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_CANCEL);
	}
	gtk_widget_show_all (dlg);
	result = gtk_dialog_run (GTK_DIALOG (dlg));
	gtk_widget_destroy (dlg);
	
    return result;
}

/* delete this some day */
gint
vazaar_cmd_strings_compare(gchar * s1, gchar * s2)
{
    gchar          *tmp1,
                   *tmp2;
	gint			i;
	
	tmp1 = g_utf8_normalize (s1, -1, G_NORMALIZE_ALL);
	tmp2 = g_utf8_normalize (s2, -1, G_NORMALIZE_ALL);
	tmp1 = g_utf8_strup (tmp1, -1);
	tmp2 = g_utf8_strup (tmp2, -1);
	if (!tmp1 || !tmp2)
		return -1;
	
	i = g_utf8_collate (s1, s2);
	
	return i;
}

void
vazaar_gui_item_content_delete(Vazaar * app)
{
    GtkWidget      *child_widget;

    child_widget = gtk_paned_get_child2(GTK_PANED(app->vpitem));
    if (child_widget) {
		g_object_ref(app->vpitem);
		gtk_container_remove(GTK_CONTAINER(app->vpitem), child_widget);
    }
}

void
vazaar_gui_expand_categories	(	Vazaar *app) {
	gtk_tree_view_expand_all (GTK_TREE_VIEW (app->treecat));
}

void
vazaar_gui_collapse_categories	(	Vazaar *app) {
	gtk_tree_view_collapse_all (GTK_TREE_VIEW (app->treecat));
}

gchar *
vazaar_cmd_get_current_date () {
	GDate	*gdate;
	gchar	*date;
	
	gdate = g_date_new ();
	g_date_set_time (gdate, time (NULL));
	date = g_strdup_printf ("%02d/%02d/%d",	g_date_get_day		(gdate), 
											g_date_get_month	(gdate), 
											g_date_get_year		(gdate));
	return date;
}

void
vazaar_gui_infopanel_activated (GtkWidget *nothing, Vazaar *this) {
	gboolean state;
	
	state = gtk_expander_get_expanded (GTK_EXPANDER (this->frameInfo));
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON(this->infoitem), !state);
}

GList *
vazaar_cmd_get_files_from_dir (const gchar *directory) {
	GnomeVFSFileInfoOptions  options;
	GnomeVFSResult			 res;
	GList					*files = NULL;
	
	options = 	GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE;
	
	gnome_vfs_directory_list_load (&files, directory, options);
	
	return files;	
}

/*
 * Dialog widget from libeog (eog-hig-dialog.c)
 */
GtkWidget*  
vazaar_hig_dialog_new (GtkWindow *window, const char *stock_id, const char *header, const char *body, gboolean modal)
{
	GtkWidget *dlg;
	GtkWidget *hbox;
	GtkWidget *image;
	GtkWidget *label;
	gint header_len;
	gint body_len;
	gint message_len;
	gchar *message;
	
	g_return_val_if_fail (stock_id != NULL, NULL);
	g_return_val_if_fail (header != NULL, NULL);
	
	dlg = gtk_widget_new	(	GTK_TYPE_DIALOG, 
			      				"border-width", 6, 
			      				"resizable", FALSE,
			      				"has-separator", FALSE,
			      				"modal", modal,
			      				"title", "",
			      				NULL);
			      
	hbox = gtk_widget_new	(	GTK_TYPE_HBOX, 
			       				"homogeneous", FALSE,
			       				"spacing", 12,
			       				"border-width", 6,
			       				NULL);

	image = gtk_widget_new	(	GTK_TYPE_IMAGE,
								"stock", stock_id,
								"icon-size", GTK_ICON_SIZE_DIALOG,
								"yalign", 0.0,
								NULL);
								
	gtk_container_add (GTK_CONTAINER (hbox), image);

	header_len = strlen (header);
	body_len = body ? strlen (body) : 0;
	message_len = header_len + body_len + 64;

	message = g_new0 (gchar, message_len);

	if (body != NULL) {
		g_snprintf (message, message_len,
			    "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s", header, body);
	}
	else {
		g_snprintf (message, message_len,
			    "<span weight=\"bold\" size=\"larger\">%s</span>", header);
	}
	
	label = gtk_label_new (message);
	g_object_set (G_OBJECT (label),
			  "selectable", TRUE,
		      "use-markup", TRUE, 
		      "wrap", TRUE,
		      "yalign", 0.0,
		      NULL); 
			       
	gtk_container_add (GTK_CONTAINER (hbox), label);
	gtk_widget_show_all (hbox);

	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), hbox);
	g_object_set (G_OBJECT (GTK_DIALOG (dlg)->vbox), "spacing", 12, NULL);

	if (window != NULL) {
		gtk_window_set_transient_for (GTK_WINDOW (dlg), window);
		gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER_ON_PARENT);
	}
	
	return dlg;
}


void vazaar_gui_first_time_wizard (Vazaar *app) {
	
}

GdkPixbuf *
get_pixbuf_from_mime_type (const gchar *mime_type, gint icon_size)
{
	GnomeIconTheme *icon_theme;
	gchar           *icon_name = NULL;
	GdkPixbuf      *icon = NULL;

	icon_theme = gnome_icon_theme_new ();

	if (icon_theme == NULL)
		return NULL;

	gnome_icon_theme_set_allow_svg (icon_theme, TRUE);

	icon_name = gnome_icon_lookup (icon_theme,
				       NULL,
				       NULL,
				       NULL,
				       NULL,
				       mime_type,
				       GNOME_ICON_LOOKUP_FLAGS_NONE,
				       NULL);
	
	if (icon_name != NULL) {
		gchar                *icon_path = NULL;
		const GnomeIconData *icon_data;
		gint                  base_size;

		icon_path = gnome_icon_theme_lookup_icon (icon_theme, 
							  icon_name,
							  icon_size,
							  &icon_data,
							  &base_size);

		if (icon_path != NULL) {
			icon = load_icon_file (icon_path, base_size, icon_size);
			g_free (icon_path);
		}
	}

	g_free (icon_name);
	g_object_unref (icon_theme);
	
	return icon;
}

/* taken from egg-recent-util.c */
static GdkPixbuf *
load_icon_file		(	gchar *filename, guint base_size, guint nominal_size)
{
	GdkPixbuf *pixbuf, *scaled_pixbuf;
        guint      width, height, size;
        double     scale;
	//g_message ("base: %d, nominal: %d", base_size, nominal_size);
	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
	if (pixbuf == NULL) {
		return NULL;
	}

	return gdk_pixbuf_scale_simple (pixbuf, nominal_size, nominal_size, GDK_INTERP_BILINEAR);
	/*
	if (base_size == 0) {
		width = gdk_pixbuf_get_width (pixbuf);
		height = gdk_pixbuf_get_height (pixbuf);
		size = MAX (width, height);
		if (size > nominal_size) {
			base_size = size;
		} else {
			// Don't scale up small icons /
			base_size = nominal_size;
		}

	} 

	if (base_size != nominal_size) {
		scale = (double) nominal_size / base_size;
		//g_message ("scale: %f", scale);
		scaled_pixbuf = scale_icon (pixbuf, &scale);
		g_object_unref (pixbuf);
		pixbuf = scaled_pixbuf;
	}

        return pixbuf;
		*/
}

GdkPixbuf *
scale_icon (GdkPixbuf *pixbuf, gint size)
{	
    return gdk_pixbuf_scale_simple (pixbuf, size, size, GDK_INTERP_BILINEAR);
}


gboolean vazaar_cmd_mime_is_cached (Vazaar *app, gchar *mime) {
	gboolean	cached = FALSE;
	GList *aux = app->mime_icon_cache;
	MIMEIconsCache	*cache;
	
	while ((aux) && (!cached)) {
		cache = (MIMEIconsCache *)aux->data;
		if ( g_utf8_collate (cache->mime, mime) == 0) {
			cached = TRUE;
		}
		aux = aux->next;
	}
	
	return cached;
}

GdkPixbuf *
load_mime_icon (Vazaar *app, gchar *mime) {
	GList *aux;
	gboolean found = FALSE;
	GdkPixbuf *icon = NULL;
	MIMEIconsCache	*cache;
	
	aux = app->mime_icon_cache;
	while ((aux) && (!found)) {
		cache = (MIMEIconsCache *)aux->data;
		if ( g_utf8_collate (cache->mime, mime) == 0) {
			icon = cache->icon;
			found = TRUE;
		}
		aux = aux->next;		
	}
	
	if (found == FALSE) {
		vazaar_cmd_mime_add_icon_to_cache (app, mime);
		icon = load_mime_icon (app, mime);
	}
	return icon;
}

void
vazaar_cmd_mime_add_icon_to_cache (Vazaar *app, gchar *mime) {
	GdkPixbuf *icon = NULL;
	MIMEIconsCache *cache = NULL;
	
	cache = g_new0 (MIMEIconsCache, 1);
	cache->mime = g_strdup (mime);
	cache->icon = get_pixbuf_from_mime_type (mime, 32);
	
	app->mime_icon_cache = g_list_append (app->mime_icon_cache, cache);
}

void vazaar_cmd_mimeiconscache_fill (Vazaar *app) {
	VazaarItem *item;
	xmlNodePtr root, child;
	gboolean	cached;
	
    root = xmlDocGetRootElement(items_doc);
    child = root->xmlChildrenNode;
    while (child != NULL) {
		item = vazaar_xml_item_get(child);
		cached = vazaar_cmd_mime_is_cached (app, item->type);
		if (!cached) {
			vazaar_cmd_mime_add_icon_to_cache (app, item->type);
		}
		child = child->next;
		g_free (item);
	}
}

gboolean 
vazaar_cmd_check_gnomevfs_result (GtkWidget *window, gint res) {
	gchar	*title,
			*message;
	
	gboolean success = TRUE;
	
	if (res != GNOME_VFS_OK) {
		title = g_strdup_printf ("Error %d", res);
		message = g_strdup_printf ("%s", gnome_vfs_result_to_string(res));
		vazaar_gui_show_dialog (window, GTK_STOCK_DIALOG_ERROR, title, message);
		g_free(title); g_free(message);
		success = FALSE;
	}
	return success;
}

void
vazaar_gui_wastebasket_toggled			(	GtkWidget *nothing, 
											Vazaar *app) {
	GtkWidget	*widget;
	gboolean	toggled;
	gint		nitems = 0;
	gchar		*str;
											
	toggled = gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON (app->wastebasket));
	/*	Se vacía el contendor que contiene el actual widget */
	widget = gtk_paned_get_child2(GTK_PANED(app->vpitem));
	if (widget) {
		g_object_ref(app->vpitem);
		gtk_container_remove(GTK_CONTAINER(app->vpitem), widget);
	}
	if (toggled) {		
		// Show items in wastebasket /
		nitems = vazaar_gui_items_load(app, "wastebasket");
		if (nitems != 0) {
			str = g_strdup_printf(_("The wastebasket contains %d items"), nitems);
			gnome_appbar_set_status(GNOME_APPBAR (app->statusbar), str);
			
			app->current_items = g_list_sort (app->current_items, (GCompareFunc)vazaar_cmd_cat_sort_items); 
			
			// Load First item in current category /
			GtkTreeSelection *item_selection = NULL;
			GList *first = g_list_first (app->current_items);
			if (first) {
				VazaarItemSort *item_sort = first->data;
				if (gtk_tree_store_iter_is_valid (GTK_TREE_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(app->treeitem))), &item_sort->iter)) {
					vazaar_gui_item_get_real(app, item_sort->item);
					item_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
					gtk_tree_selection_select_iter (item_selection, &item_sort->iter);	
				}
				else {
					//g_message ("You should not never see this message");
				}
			}	
			
			gtk_widget_show (app->frmwastebasket);		
		}
		else {
			gchar *title = g_strdup_printf (_("The wastebasket is empty"));
			gnome_appbar_set_status(GNOME_APPBAR (app->statusbar), title);
		}
	}
	else {
		on_trvVazaarCat_cursor_changed (NULL, app);
		gtk_widget_hide (app->frmwastebasket);
	}
}

void
vazaar_gui_fullscreen_mode 			(	GtkWidget *nothing, 
										Vazaar *app) {
	gboolean	toggled;
											
	toggled = gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON (app->fullscreen));
	if (toggled) {
		gtk_window_fullscreen (GTK_WINDOW (app->window));
	}
	else {
		gtk_window_unfullscreen (GTK_WINDOW (app->window));
	}
	vazaar_gui_toolbar_items_toggle (app);
}

void
vazaar_gui_emptybasket_clicked			(	GtkWidget *nothing, 
											Vazaar *app) {
	gint			result;
	gchar			*title,
					*message;
    GtkTreeIter     parent = { 0 };
    xmlNodePtr      root,
                    child;
    VazaarItem		*item;
	GList			*delete_items = NULL;
	title = g_strdup_printf (_("Are you sure?"));
	message = g_strdup_printf (_("All items in the wastebasket will be deleted"));												
	result = vazaar_gui_show_dialog (app->window, GTK_STOCK_DIALOG_WARNING, title, message);
	switch (result) {
	case GTK_RESPONSE_OK:
		item = (VazaarItem *) NULL;
	
		root = xmlDocGetRootElement(items_doc);
		child = root->xmlChildrenNode;
		while (child != NULL) {
			item = vazaar_xml_item_get(child);
			if (vazaar_cmd_item_belongto("wastebasket", item->categories)) {
				delete_items = g_list_append (delete_items, item);
			}
			child = child->next;
		}	
		GList *aux = delete_items;
		while (aux) {
			item = (VazaarItem *)aux->data;			
			vazaar_cmd_item_delete (item->id);
			aux = aux->next;
		}
		vazaar_gui_wastebasket_toggled (NULL, app);

		break;
	case GTK_RESPONSE_CANCEL:
		break;
	}										
}

gboolean vazaar_gui_category_is_wastebasket (Vazaar *app) {
	gchar *category = vazaar_gui_get_current_category (app);
	gint res;
	
	if (category) {
		res = vazaar_cmd_strings_compare (category, "wastebasket");
		if (res == 0) {
			return TRUE;
		}
	}	
	return FALSE;
}

gchar *
vazaar_gui_get_current_category			(	Vazaar	*app) {
    GtkTreeSelection	*selection;
    GtkTreeModel		*model;
    GtkTreeIter			 iter;
    gchar				*name;

    /*
     * This will only work in single or browse selection mode! 
     */
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treecat));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get	(	model, &iter, 
				   				ITEM_COL_NOM, &name, 
								-1);
		return name;
	}
	
	return NULL;
}

gchar *
vazaar_gui_get_current_item				(	Vazaar	*app) {
    GtkTreeSelection	*selection;
    GtkTreeModel		*model;
    GtkTreeIter			 iter;
    gchar				*name;

    /*
     * This will only work in single or browse selection mode! 
     */
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get	(	model, &iter, 
				   				ITEM_COL_NOM, &name, 
								-1);
		return name;
	}
	
	return NULL;
}

void 
vazaar_gui_toolbar_items_toggle (Vazaar *app) {
	gboolean toggled;
	
	toggled = gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON (app->fullscreen));
	gtk_widget_set_sensitive (app->addcat, !toggled);
	gtk_widget_set_sensitive (app->editcat, !toggled);
	gtk_widget_set_sensitive (app->delcat, !toggled);
	//gtk_widget_set_sensitive (app->expcat, !toggled);
	//gtk_widget_set_sensitive (app->collcat, !toggled);
	//gtk_widget_set_sensitive (app->wastebasket, !toggled);
	//gtk_widget_set_sensitive (app->emptybasket, !toggled);
    gtk_widget_set_sensitive (app->additem, !toggled);
    gtk_widget_set_sensitive (app->edititem, !toggled);
    gtk_widget_set_sensitive (app->delitem, !toggled);
	//gtk_widget_set_sensitive (app->infoitem, !toggled);
	//gtk_widget_set_sensitive (app->firstitem, !toggled);
	//gtk_widget_set_sensitive (app->previtem, !toggled);
	//gtk_widget_set_sensitive (app->refreshitem, !toggled);
	//gtk_widget_set_sensitive (app->nextitem, !toggled);
	//gtk_widget_set_sensitive (app->lastitem, !toggled);
	//gtk_widget_set_sensitive (app->viewcat,  !toggled);
}
